//+------------------------------------------------------------------+
//|                                         BandPassIndicator_v1.mq4 |
//|                           Copyright  2007, TrendLaboratory Ltd. |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, TrendLaboratory Ltd."
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"
//---- indicator settings
#property indicator_separate_window 
#property indicator_buffers 2 
#property indicator_color1 LightBlue 
#property indicator_color2 Tomato 
#property indicator_width1 2
#property indicator_width2 1
//---- indicator parameters
extern int     Price          = 0;  //Price mode : 0-Close,1-Open,2-High,3-Low,4-Median,5-Typical,6-Weighted
extern int     Order          = 3;  //Filter Order: 1-EMA,2-2nd Order,3-3rd Order
extern int     FastPeriod     = 6;  //Fast Filter Period 
extern int     SlowPeriod     = 30; //Slow Filter Period 
extern int     SignalPeriod   = 8;  //Signal Period 
extern int     PreSmooth      = 3;  //Pre-smoothing period
extern int     PreSmoothMode  = 3;  //Pre-smoothing MA Mode: 0-SMA,1-EMA,2-SMMA,3-LWMA
//---- indicator buffers
double     BandPass[];
double     Signal[];
double     Smoother[];
double     Filter1[];
double     Filter2[];

int        draw_begin;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicator buffers mapping
   IndicatorBuffers(5);
   SetIndexBuffer(0,BandPass);
   SetIndexBuffer(1,Signal);
   SetIndexBuffer(2,Smoother);
   SetIndexBuffer(3,Filter1);
   SetIndexBuffer(4,Filter2);  
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);
   draw_begin = SlowPeriod + PreSmooth + SignalPeriod+3;
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   //IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("BandPassIndicator("+Order+","+FastPeriod+","+SlowPeriod+")");
   SetIndexLabel(0,"BandPass");
   SetIndexLabel(1,"Signal");
//---- initialization done
   return(0);
}
//+------------------------------------------------------------------+
//| BandPassIndicator_v1                                                 |
//+------------------------------------------------------------------+
int start()
{
   int limit, i, shift;
   int counted_bars=IndicatorCounted();
   double a1, b1, c1, a2, b2, c2, a3, b3, c3, pi = 3.1415926535;
   
   if(counted_bars<1)
   for(i=1;i<=draw_begin;i++) 
   {
   BandPass[Bars-i]=0; 
   Signal[Bars-i]=0; 
   Filter1[Bars-i]=iMA(NULL,0,1,0,PreSmoothMode,Price,Bars-i); 
   Filter2[Bars-i]=iMA(NULL,0,1,0,PreSmoothMode,Price,Bars-i);
   }
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

   for(shift=limit; shift>=0; shift--)
   Smoother[shift] = iMA(NULL,0,PreSmooth,0,PreSmoothMode,Price,shift);
   
   for(shift=limit; shift>=0; shift--)
   {
      if(Order == 1) 
      {
      Filter1[shift] = iMAOnArray(Smoother,0,FastPeriod,0,1,shift);
      Filter2[shift] = iMAOnArray(Smoother,0,SlowPeriod,0,1,shift);
      }
      else
		if(Order == 2) 
		{
		a1 = MathExp(-MathSqrt(2)*pi/FastPeriod);
		b1 = 2*a1*MathCos(MathSqrt(2)*pi/FastPeriod);
    	a2 = MathExp(-MathSqrt(2)*pi/SlowPeriod);
		b2 = 2*a2*MathCos(MathSqrt(2)*pi/SlowPeriod);
    
    	Filter1[shift] = b1*Filter1[shift+1] - a1*a1*Filter1[shift+2] + (1 - b1 + a1*a1)*Smoother[shift];
		Filter2[shift] = b2*Filter2[shift+1] - a2*a2*Filter2[shift+2] + (1 - b2 + a2*a2)*Smoother[shift];
		}
		else
		if(Order == 3) 
		{
		a1 = MathExp(-pi/FastPeriod);
		b1 = 2*a1*MathCos(MathSqrt(3)*pi/FastPeriod);
    	c1 = MathExp(-2*pi/FastPeriod); 
    	a2 = MathExp(-pi/SlowPeriod);
		b2 = 2*a2*MathCos(MathSqrt(3)*pi/SlowPeriod);
    	c2 = MathExp(-2*pi/SlowPeriod); 
    	
    	Filter1[shift] = (b1+c1)*Filter1[shift+1] - (c1+b1*c1)*Filter1[shift+2] + c1*c1*Filter1[shift+3] + (1-b1+c1)*(1-c1)*Smoother[shift];
		Filter2[shift] = (b2+c2)*Filter2[shift+1] - (c2+b2*c2)*Filter2[shift+2] + c2*c2*Filter2[shift+3] + (1-b2+c2)*(1-c2)*Smoother[shift];
		}
		
   BandPass[shift] = Filter1[shift] - Filter2[shift];
   }
   
   for(shift=limit; shift>=0; shift--)
   {
      if(Order == 1) 
      {
      Signal[shift] = Signal[shift+1] + 2.0/(1+SignalPeriod)*(BandPass[shift] - Signal[shift+1]);  
      }
      else
		if(Order == 2) 
		{
		a3 = MathExp(-MathSqrt(2)*pi/SignalPeriod);
		b3 = 2*a3*MathCos(MathSqrt(2)*pi/SignalPeriod);
    	Signal[shift] = b3*Signal[shift+1] - a3*a3*Signal[shift+2] + (1 - b3 + a3*a3)*BandPass[shift];
		}
		else
		if(Order == 3) 
		{
		a3 = MathExp(-pi/SignalPeriod);
		b3 = 2*a3*MathCos(MathSqrt(3)*pi/SignalPeriod);
    	c3 = MathExp(-2*pi/SignalPeriod);  
    	Signal[shift] = (b3+c3)*Signal[shift+1] - (c3+b3*c3)*Signal[shift+2] + c3*c3*Signal[shift+3] + (1-b3+c3)*(1-c3)*BandPass[shift];
		}
   
   }
//---- done
   return(0);
}